'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.resolveOnClient = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _reactRouter = require('react-router');

var _helpers = require('./helpers');

var _locationStorage = require('./locationStorage');

var resolveOnClient = exports.resolveOnClient = function resolveOnClient(history, routes, _ref) {
  var getState = _ref.getState;
  var dispatch = _ref.dispatch;
  var custom = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

  var beforeTransitionHooks = [];
  var afterTransitionHooks = [];

  var locationStorage = (0, _locationStorage.createLocationStorage)();

  var transitonRule = function transitonRule(oldLoc, newLoc) {
    return oldLoc.pathname !== newLoc.pathname || oldLoc.search !== newLoc.search;
  };

  var beforeTransition = function beforeTransition(callback) {
    if (beforeTransitionHooks.indexOf(callback) === -1) {
      beforeTransitionHooks.push(callback);
    }
  };

  var afterTransition = function afterTransition(callback) {
    if (afterTransitionHooks.indexOf(callback) === -1) {
      afterTransitionHooks.push(callback);
    }
  };

  function setTransitionRule(rule) {
    transitonRule = rule;
    return transitonRule;
  }

  var executeBeforeTransition = function executeBeforeTransition(attrs) {
    beforeTransitionHooks.forEach(function (callback) {
      return callback(attrs);
    });
  };

  var executeAfterTransition = function executeAfterTransition(attrs) {
    afterTransitionHooks.forEach(function (callback) {
      return callback(attrs);
    });
  };

  var stopResolving = history.listenBefore(function (location, continueTransition) {
    var lastLocation = locationStorage.getLastLocation();
    locationStorage.setNewLocation(location);
    if (!transitonRule(lastLocation, location)) return;

    (0, _reactRouter.match)({ history: history, location: location, routes: routes }, function (error, redirectLocation, renderProps) {
      if (redirectLocation) {
        history.transitionTo(redirectLocation);
      } else if (renderProps) {
        (function () {
          var components = renderProps.components;
          var params = renderProps.params;

          var attrs = _extends({}, custom, { location: location, params: params, getState: getState, dispatch: dispatch });
          executeBeforeTransition(attrs);
          var deferResolve = function deferResolve() {
            return Promise.all((0, _helpers.getDeferResolveDependencies)(components)(attrs));
          };
          Promise.all((0, _helpers.getPreResolveDependencies)(components)(attrs)).then(continueTransition, continueTransition).then(executeAfterTransition.bind(undefined, attrs), executeAfterTransition.bind(undefined, attrs)).then(deferResolve, deferResolve);
        })();
      }
    });
  });

  return {
    beforeTransition: beforeTransition,
    afterTransition: afterTransition,
    setTransitionRule: setTransitionRule,
    stopResolving: stopResolving
  };
};